{
 TExitWin version 1.0 copyright  1996 by Alexander Meeder
 Encapsulates the Win32 API "ExitWindowsEx' function
 Very easy to use, just place it on your form, choose the appropriate action
 with the ExitWinType-property, call the ExitWin1.Execute function which returns
 false if something went wrong. You can then use the LastError property

 please mail me if you use it or have some suggestions: ameeder@dds.nl
}

unit ExitWin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  // Different options to Shutdown/Logoff Windows95/NT
  TExitWinType = (ewLogoff, ewReboot, ewShutDown, ewPowerOff, ewForce);

  // New component, encapsulates Win32 API ExitWindowsEx
  TExitWin = class(TComponent)
  private
    { Private declarations }
    FExitWinType: TExitWinType;
    FLastError: string;

    procedure SetExitWinType(Value: TExitWinType);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property ExitWinType: TExitWinType read FExitWinType write SetExitWinType default EWReboot;
    property LastError: string read FLastError;
  end;

procedure Register;

implementation

// Override 'create-method'
constructor TExitWin.Create(AOwner: TComponent);
begin
  inherited;
  FExitWinType := ewLogoff;
end;

// Override 'destroy-method'
destructor TExitWin.Destroy;
begin
  inherited;
end;

// Property ExitWinType procedure
procedure TExitWin.SetExitWinType(Value: TExitWinType);
begin
  if Value <> FExitWinType then
  begin
    FExitWinType := Value;
  end;
end;

// The functionality of this component is build in this routine
function TExitWin.Execute: boolean;
begin
  case FExitWinType of
       ewLogoff   : Result := ExitWindowsEx(EWX_LOGOFF, 0);
       ewReboot   : Result := ExitWindowsEx(EWX_REBOOT, 0);
       ewShutDown : Result := ExitWindowsEx(EWX_SHUTDOWN, 0);
       ewPowerOff : Result := ExitWindowsEx(EWX_POWEROFF, 0);
       ewForce    : Result := ExitWindowsEx(EWX_FORCE, 0);
  end;
  // If somthing went wrong, set property 'LastError'
  if not Result then
     FLastError := 'Error: ' + IntToStr(GetLastError);
end;

procedure Register;
begin
  RegisterComponents('Win95', [TExitWin]);
end;

end.
