{ TIEButton component version 1.2, copyright  A. Meeder, 1996

  ** It's freeware, so you don't have to pay me, but I do not guarantee the working of it

  This is the first component I made, before this I've only 'changed' some components. The idea for
  TIEButton came from Microsoft's Internet Explorer 3.0 (this is where 'IE' stands for). The buttons
  on the button/task-bar behave different from normal 'standard' Windows-buttons.

  If you have suggestions about TIEButton or make changes, please mail me: ameeder@dds.nl

  1.2 repaired a problem with the borders, thank to Ron Manske nova@q.continuum.net
}

unit IEButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls;

type
  // Special types for several properties
  TButtonLayout = (blCaptionCenter, blCaptionLeft, blCaptionRight, blCaptionTop, blCaptionBottom);
  TBorderWidth = 1..10;
  TMargin = 3..25;
  TNumGlyphs = 1..3;

  // The new type TIEButton, derived from TGraphicControl
  TIEButton = class(TGraphicControl)
  private
    { Private declarations }
    FBorderWidth: TBorderWidth;
    FButtonLayout: TButtonLayout;
    FCaption: TCaption;
    FGlyph: TBitmap;
    FMargin: TMargin;
    FNumGlyphs: TNumGlyphs;
    FTransparentColor: TColor;

    BtnRect: TRect;
    HasMouse: boolean;

    procedure SetBorderWidth(Value: TBorderWidth);
    procedure SetButtonLayOut(Value: TButtonLayout);
    procedure SetGlyph(Value: TBitmap);
    procedure SetMargin(Value: TMargin);
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure SetText(Value: TCaption);
    procedure SetTransparentColor(Value: TColor);
    procedure InitBtnRect;
    procedure DrawCaption;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;  // New event
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;  // New event
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    // New and inherited properties and events
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 1;
    property ButtonLayout: TButtonLayout read FButtonLayout write SetButtonLayout default blCaptionCenter;
    property Caption: TCaption read FCaption write SetText;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Margin: TMargin read FMargin write SetMargin default 5;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs default 2;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    property Color;
    property Enabled;
    property Font;
    property Name;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

// Initialize BtnRect
procedure TIEButton.InitBtnRect;
begin
  BtnRect := GetClientRect;
end;

// Function to place the caption (center, left, right, top, bottom)
procedure TIEButton.DrawCaption;
var
  TopLeft: TPoint;
begin

  // Initialize Brush-/Font-settings (bsClear so Caption is transparent
  with Canvas do
  begin
    Font.Color := Self.Font.Color;
    Font.Name := Self.Font.Name;
    Font.Size := Self.Font.Size;
    Font.Style := Self.Font.Style;
    Brush.Style := bsClear;
  end;

  // Define the exact placement of the Caption on the button
  case ButtonLayout of
    blCaptionCenter: begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
    blCaptionTop:    begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := FMargin;
                     end;
    blCaptionBottom: begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) - FMargin;
                     end;
    blCaptionLeft:   begin
                       TopLeft.X := FMargin;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
    blCaptionRight:  begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) - FMargin;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
  end;

  // Is there a caption to draw?
  if Caption <> '' then
  with Canvas do
  begin
     // If button is enabled then draw caption just one time
     if Enabled then
        TextOut(TopLeft.X, TopLeft.Y, Caption)
     else
     // If button is disabled then draw caption to times (Gray with White-shade)
     begin
       Font.Color := clWhite;
       TextOut(TopLeft.X + 1, TopLeft.Y + 1, Caption);
       Font.Color := clGrayText;
       TextOut(TopLeft.X, TopLeft.Y, Caption);
     end;

  end;
end;

// Border-property 'write' method
procedure TIEButton.SetBorderWidth(Value: TBorderWidth);
begin
  if (Value <> FBorderWidth) and (Value >= 1) and (Value < 10) then
  begin
    FBorderWidth := Value;
    SetMargin(Value + 2);  // Margin alway 2 more then BorderWidth
    invalidate;
  end;
end;

// ButtonLayout-property 'write' method
procedure TIEButton.SetButtonLayout(Value: TButtonLayout);
begin
  if Value <> FButtonLayout then
  begin
     FButtonLayout := Value;
     invalidate;
  end;
end;

// Glyph-property 'write' method
procedure TIEButton.SetGlyph(Value: TBitmap);
begin
  FGlyph.Assign(Value);
end;

// Margin-property 'write' method
procedure TIEButton.SetMargin(Value: TMargin);
begin
  if (Value <> FMargin) and (Value > FBorderWidth + 1) then
  begin
    FMargin := Value;
    invalidate
  end;
end;

// NumGlyphs-property 'write' method
procedure TIEButton.SetNumGlyphs(Value: TNumGlyphs);
begin
  if (Value <> FNumGlyphs) and (Value <= 3) and (Value > 0) then
  begin
    FNumGlyphs := Value;
    invalidate;
  end;
end;

// Caption-property 'write' method
procedure TIEButton.SetText(Value: TCaption);
begin
  if Value <> FCaption then FCaption := Value;
  invalidate;
end;

// Transparent-property 'write' method
procedure TIEButton.SetTransparentColor(Value: TColor);
begin
  if Value <> FTransparentColor then
  begin
    FTransparentColor := Value;
    invalidate;
  end;
end;

// Initialize standard behavior/layout of IEButton
constructor TIEButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BorderWidth := 1;
  ButtonLayout := blCaptionCenter;
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clBtnFace;
  FGlyph := TBitmap.Create;
  Font := Canvas.Font;
  HasMouse := false;
  Height := 25;
  Margin := 5;
  FNumGlyphs := 2;
  Width := 25;
end;

// Destroy IEButton, free the memory used by FGlyph
destructor TIEButton.Destroy;
begin
  FGlyph.Free;
  inherited Destroy;
end;

// New event, when the mouse enters this component, draw Frame3D
procedure TIEButton.CMMouseEnter(var Message: TMessage);
begin
  HasMouse := true;
  if Enabled then
  begin
    InitBtnRect;
    invalidate;
    Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth);
  end;
end;

// New event, when the mouse leaves this component delete frame by drawing Frame3D with buttonface color
procedure TIEButton.CMMouseLeave(var Message: TMessage);
begin
  HasMouse := false;
  if Enabled then
  begin
    InitBtnRect;
    invalidate;
    Frame3D(Canvas, BtnRect, Color, Color, FBorderWidth);
  end;
end;

// Draw button on Canvas depending on several properties
procedure TIEButton.Paint;
var
  GlyphRect, BitmapRect: TRect;
  TopLeft: TPoint;
  PartGlyphWidth: integer;
begin
  InitBtnRect;

  // Initialize Brush-/Pen-settings
  with Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := Color;
    Pen.Color := clBtnFace;
  end;

  // Draw background (face) of button
  Canvas.Rectangle(0, 0, Width, Height);

  // Draw Frame3D in designtime so we can manage the button easily
  if (csDesigning in ComponentState) then
  begin
    InitBtnRect;
    Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth);
  end;

    // Set the Glyph depending the placement of the Caption
    case ButtonLayout of
      blCaptionCenter: begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
      blCaptionTop:    begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := (Self.Height - FGlyph.Height) - Margin;
                       end;
      blCaptionBottom: begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := Margin;
                       end;
      blCaptionLeft:   begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) - Margin;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
      blCaptionRight:  begin
                         TopLeft.X := Margin;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
    end;

  // Set Glyph Rect on Canvas, depending on placement of Caption
  GlyphRect := Rect(TopLeft.X, TopLeft.Y, TopLeft.X + (FGlyph.Width div NumGlyphs), TopLeft.Y + FGlyph.Height);

  PartGlyphWidth := FGlyph.Width div NumGlyphs;

  // Set params BitmapRect depending on Enabled/Disabled control and NumGlyphs
  if Enabled and HasMouse then
  begin
     BitmapRect := Rect(0, 0, PartGlyphWidth, FGlyph.Height);
     Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth);
  end;
  if Enabled and not HasMouse then
  begin
     BitmapRect := Rect(FGlyph.Width - PartGlyphWidth, 0, PartGlyphWidth * NumGlyphs, FGlyph.Height);
     Frame3D(Canvas, BtnRect, Color, Color, FBorderWidth);
  end;
  if not Enabled and (FNumGlyphs > 1) then
     BitmapRect := Rect(PartGlyphWidth, 0, PartGlyphWidth * 2, FGlyph.Height);
  if not Enabled and (FNumGlyphs = 1) then
     BitmapRect := Rect(0, 0, PartGlyphWidth, FGlyph.Height);

  // Draw FGlyph on buttoncanvas
  Canvas.BrushCopy(GlyphRect, FGlyph, BitmapRect, TransparentColor);

  // Call DrawCaption finaly so that its drawn on top
  DrawCaption;
end;

// Override standard MouseDown event to simulate border UpDown-state
procedure TIEButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled then
  begin
    InitBtnRect;
    Frame3D(Canvas, BtnRect, clBtnShadow, clBtnHighLight, FBorderWidth);
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

// Override standard MouseUp event to simulate border UpDown-state
procedure TIEButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled then
  begin
    InitBtnRect;
    if HasMouse then
       Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth)
    else
       Frame3D(Canvas, BtnRect, Color, Color, FBorderWidth);
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

// Register this 'MS Internet Explorer 3.0' like button on the Win95 tab
procedure Register;
begin
  RegisterComponents('Win95', [TIEButton]);
end;

end.